@@ -23,8 +23,8 @@ setClass("RandomPars",
                         est='logical',
                         between='logical',
                         parnum='numeric',
-                        ndim='integer',  
-                        gframe='data.frame',                        
+                        ndim='integer',
+                        gframe='data.frame',
                         gdesign='matrix',
                         lbound='numeric',
                         ubound='numeric',
@@ -49,7 +49,6 @@ setClass("AllItemsClass",
                         fixed.design='matrix',
                         dat='matrix',
                         ncat='integer',
-                        rs='matrix',
                         gradient='numeric',
                         hessian='matrix',
                         itemtrace='matrix',
@@ -57,7 +56,7 @@ setClass("AllItemsClass",
                         ubound='numeric',
                         any.prior='logical',
                         prior.type='character',
-                        prior_1='numeric',                        
+                        prior_1='numeric',
                         prior_2='numeric',
                         'VIRTUAL')
 )
