@@ -2,6 +2,8 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, DERIV)
 {
     verbose <- list$verbose
     nfact <- list$nfact
+    if(list$EH && nfact != 1L)
+        stop('empirical histogram only available for unidimensional models')
     NCYCLES <- list$NCYCLES
     TOL <- list$TOL
     MSTEPTOL <- list$MSTEPTOL
@@ -44,7 +46,7 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, DERIV)
         }
     }
     stagecycle <- 1L
-    converge <- 1L    
+    converge <- 1L
     inverse_fail_count <- 1L
     ##
     L <- c()
@@ -92,49 +94,61 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, DERIV)
            else est <- c(est, rep(FALSE, length(pars[[g]][[j]]@est)))
        }
     }
+    groupest <- FALSE
+    for(g in 1L:ngroups)
+        groupest <- any(groupest, pars[[g]][[J+1]]@est)
     if(length(constrain) > 0L)
        for(i in 1L:length(constrain))
            est[constrain[[i]][-1L]] <- FALSE
     names(longpars) <- names(est)
     EMhistory <- matrix(NA, NCYCLES+1L, length(longpars))
-    EMhistory[1L,] <- longpars    
+    EMhistory[1L,] <- longpars
     gTheta <- vector('list', ngroups)
     ANY.PRIOR <- rep(FALSE, ngroups)
     NO.CUSTOM <- !any(sapply(pars[[1L]], class) %in% 'custom')
     for(g in 1L:ngroups){
-        gTheta[[g]] <- Theta 
+        gTheta[[g]] <- Theta
         if(length(prodlist) > 0L)
             gTheta[[g]] <- prodterms(gTheta[[g]],prodlist)
         if(BFACTOR){
             Thetabetween <- thetaComb(theta=theta, nfact=nfact-ncol(sitems))
             prior[[g]] <- dnorm(theta, 0, 1)
-            prior[[g]] <- prior[[g]]/sum(prior[[g]])  
+            prior[[g]] <- prior[[g]]/sum(prior[[g]])
         }
         ANY.PRIOR[g] <- any(sapply(pars[[g]], function(x) x@any.prior))
-    }    
+    }
     preMstep.longpars2 <- preMstep.longpars <- longpars
     accel <- 0
-    
+    Estep.time <- Mstep.time <- 0
+    collectLL <- rep(NA, NCYCLES)
+
     #EM
     for (cycles in 1L:NCYCLES){
         #priors
-        for(g in 1L:ngroups){            
-            gstructgrouppars[[g]] <- ExtractGroupPars(pars[[g]][[J+1L]])            
+        start <- proc.time()[3L]
+        for(g in 1L:ngroups){
+            gstructgrouppars[[g]] <- ExtractGroupPars(pars[[g]][[J+1L]])
             if(BFACTOR){
                 sel <- 1L:(nfact-ncol(sitems))
                 Priorbetween[[g]] <- mvtnorm::dmvnorm(Thetabetween,
                                         gstructgrouppars[[g]]$gmeans[sel],
-                                        gstructgrouppars[[g]]$gcov[sel,sel,drop=FALSE])                
-                Priorbetween[[g]] <- Priorbetween[[g]]/sum(Priorbetween[[g]]) 
-                Prior[[g]] <- apply(expand.grid(Priorbetween[[g]], prior[[g]]), 1, prod)                
+                                        gstructgrouppars[[g]]$gcov[sel,sel,drop=FALSE])
+                Priorbetween[[g]] <- Priorbetween[[g]]/sum(Priorbetween[[g]])
+                Prior[[g]] <- apply(expand.grid(Priorbetween[[g]], prior[[g]]), 1, prod)
                 next
             }
             Prior[[g]] <- mvtnorm::dmvnorm(gTheta[[g]][ ,1L:nfact,drop=FALSE],gstructgrouppars[[g]]$gmeans,
                                            gstructgrouppars[[g]]$gcov)
-            Prior[[g]] <- Prior[[g]]/sum(Prior[[g]])                                              
+            Prior[[g]] <- Prior[[g]]/sum(Prior[[g]])
+        }
+        if(list$EH && cycles > 1L){
+            for(g in 1L:ngroups)
+                Prior[[g]] <- rowSums(rlist[[g]][[1L]]) / sum(rlist[[g]][[1L]])
+        } else if(!is.null(list$customPriorFun)){
+            for(g in 1L:ngroups)
+                Prior[[g]] <- list$customPriorFun(gTheta[[g]])
         }
         #Estep
-        lastLL <- LL
         LL <- 0
         for(g in 1L:ngroups){
             if(BFACTOR){
@@ -148,36 +162,40 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, DERIV)
             }
             LL <- LL + sum(r[[g]]*log(rlist[[g]]$expected))
         }
+        collectLL[cycles] <- LL
         for(g in 1L:ngroups){
             for(i in 1L:J){
                 tmp <- c(itemloc[i]:(itemloc[i+1L] - 1L))
-                pars[[g]][[i]]@rs <- rlist[[g]]$r1[, tmp]
+                pars[[g]][[i]]@dat <- rlist[[g]]$r1[, tmp]
             }
         }
+        Estep.time <- Estep.time + proc.time()[3L] - start
+        start <- proc.time()[3L]
         preMstep.longpars2 <- preMstep.longpars
         preMstep.longpars <- longpars
-        if(all(!est)) break
+        if(all(!est) && !groupest) break
         longpars <- Mstep(pars=pars, est=est, longpars=longpars, ngroups=ngroups, J=J,
-                          gTheta=gTheta, itemloc=itemloc, Prior=Prior, ANY.PRIOR=ANY.PRIOR, 
+                          gTheta=gTheta, itemloc=itemloc, Prior=Prior, ANY.PRIOR=ANY.PRIOR,
                           NO.CUSTOM=NO.CUSTOM, PrepList=PrepList, L=L, UBOUND=UBOUND, LBOUND=LBOUND,
-                          rlist=rlist, constrain=constrain, cycle=cycles, DERIV=DERIV)                   
-        if(list$accelerate && cycles > 10L && cycles %% 3 == 0L){            
+                          rlist=rlist, constrain=constrain, cycle=cycles, DERIV=DERIV)
+        if(list$accelerate && cycles > 10L && cycles %% 3 == 0L){
             dX2 <- preMstep.longpars - preMstep.longpars2
             dX <- longpars - preMstep.longpars
             d2X2 <- dX - dX2
             accel <- 1 - sqrt((dX %*% dX) / (d2X2 %*% d2X2))
             if(accel < -5) accel <- -5
             longpars <- (1 - accel) * longpars + accel * preMstep.longpars
-        }        
+        }
         pars <- reloadPars(longpars=longpars, pars=pars, ngroups=ngroups, J=J)
         EMhistory[cycles+1L,] <- longpars
         if(verbose)
             cat(sprintf('\rIteration: %d, Log-Lik: %.3f, Max-Change: %.5f',
                         cycles, LL, max(abs(preMstep.longpars - longpars))))
-        if(cycles > 3L && all(abs(preMstep.longpars - longpars) < TOL))  break
+        if(all(abs(preMstep.longpars - longpars) < TOL))  break
+        Mstep.time <- Mstep.time + proc.time()[3L] - start
     } #END EM
     if(cycles == NCYCLES){
-        message('EM iterations terminated after ', cycles, ' iterations.')        
+        message('EM iterations terminated after ', cycles, ' iterations.')
         converge <- 0L
     }
     infological <- estpars & !redun_constr
@@ -189,7 +207,7 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, DERIV)
         ind1 <- 1L
         for(group in 1L:ngroups){
             for (i in 1L:J){
-                deriv <- Deriv(x=pars[[group]][[i]], Theta=gTheta[[g]], EM = TRUE, 
+                deriv <- Deriv(x=pars[[group]][[i]], Theta=gTheta[[g]], EM = TRUE,
                                estHess=TRUE)
                 ind2 <- ind1 + length(deriv$grad) - 1L
                 h[ind1:ind2, ind1:ind2] <- pars[[group]][[i]]@hessian <- deriv$hess
@@ -208,18 +226,20 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, DERIV)
         #return internal functions for SEM.SE
         return(list(pars=pars, cycles = cycles, info=matrix(0), longpars=longpars, converge=converge,
                     logLik=LL, rlist=rlist, SElogLik=0, L=L, infological=infological,
-                    estindex_unique=estindex_unique, correction=correction, hess=hess,
+                    estindex_unique=estindex_unique, correction=correction, hess=hess, Prior=Prior,
                     estpars=estpars & !redun_constr, redun_constr=redun_constr, ngroups=ngroups,
-                    LBOUND=LBOUND, UBOUND=UBOUND, EMhistory=na.omit(EMhistory), random=list()))
+                    LBOUND=LBOUND, UBOUND=UBOUND, EMhistory=na.omit(EMhistory), random=list(),
+                    time=c(Estep=as.numeric(Estep.time), Mstep=as.numeric(Mstep.time)), collectLL=na.omit(collectLL)))
     }
     ret <- list(pars=pars, cycles = cycles, info=matrix(0), longpars=longpars, converge=converge,
                 logLik=LL, rlist=rlist, SElogLik=0, L=L, infological=infological,
-                estindex_unique=estindex_unique, correction=correction, hess=hess, random=list())
+                estindex_unique=estindex_unique, correction=correction, hess=hess, random=list(),
+                Prior=Prior, time=c(Estep=as.numeric(Estep.time), Mstep=as.numeric(Mstep.time)))
     ret
 }
 
 # Estep for mirt
-Estep.mirt <- function(pars, tabdata, Theta, prior, itemloc, NO.CUSTOM=FALSE, 
+Estep.mirt <- function(pars, tabdata, Theta, prior, itemloc, NO.CUSTOM=FALSE,
                        itemtrace=NULL, deriv = FALSE)
 {
     nquad <- nrow(Theta)
@@ -234,7 +254,7 @@ Estep.mirt <- function(pars, tabdata, Theta, prior, itemloc, NO.CUSTOM=FALSE,
 }
 
 # Estep for bfactor
-Estep.bfactor <- function(pars, tabdata, Theta, prior, Prior, Priorbetween, specific, 
+Estep.bfactor <- function(pars, tabdata, Theta, prior, Prior, Priorbetween, specific,
                           NO.CUSTOM=FALSE, sitems, itemloc, itemtrace=NULL)
 {
     J <- length(itemloc) - 1L
@@ -248,15 +268,15 @@ Estep.bfactor <- function(pars, tabdata, Theta, prior, Prior, Priorbetween, spec
         r1[ ,itemloc[i]:(itemloc[i+1L]-1L)] <-
             retlist$r1[ ,itemloc[i]:(itemloc[i+1L]-1L) + (specific[i] - 1L)*ncol(X) ]
     }
-    r1 <- r1 * Prior 
+    r1 <- r1 * Prior
     return(list(r1=r1, expected=retlist$expected))
 }
 
 Mstep <- function(pars, est, longpars, ngroups, J, gTheta, itemloc, PrepList, L, ANY.PRIOR,
                   UBOUND, LBOUND, constrain, cycle, DERIV, Prior, rlist, NO.CUSTOM){
     p <- longpars[est]
-    opt <- try(optim(p, fn=Mstep.LL, gr=Mstep.grad, method='L-BFGS-B', 
-                     control=list(maxit=ifelse(cycle > 10L, 10L, 5L), fnscale = -1L),  
+    opt <- try(optim(p, fn=Mstep.LL, gr=Mstep.grad, method='L-BFGS-B',
+                     control=list(maxit=ifelse(cycle > 10L, 10L, 5L), fnscale = -1L),
                      DERIV=DERIV, rlist=rlist, NO.CUSTOM=NO.CUSTOM,
                      est=est, longpars=longpars, pars=pars, ngroups=ngroups, J=J, gTheta=gTheta,
                      PrepList=PrepList, L=L, constrain=constrain, ANY.PRIOR=ANY.PRIOR,
@@ -271,7 +291,7 @@ Mstep <- function(pars, est, longpars, ngroups, J, gTheta, itemloc, PrepList, L,
 #                silent=TRUE)
     if(is(opt, 'try-error'))
         stop(opt)
-    longpars[est] <- opt$par    
+    longpars[est] <- opt$par
     if(length(constrain) > 0L)
         for(i in 1L:length(constrain))
             longpars[constrain[[i]][-1L]] <- longpars[constrain[[i]][1L]]
@@ -296,8 +316,8 @@ Mstep.LL <- function(p, est, longpars, pars, ngroups, J, gTheta, PrepList, L, NO
     pars <- reloadPars(longpars=longpars, pars=pars, ngroups=ngroups, J=J)
     LLs <- numeric(ngroups)
     for(g in 1L:ngroups)
-        LLs[g] <- LogLikMstep(pars[[g]], Theta=gTheta[[g]], rs=rlist[[g]], 
-                              itemloc=itemloc, NO.CUSTOM=NO.CUSTOM, any.prior=ANY.PRIOR[g])        
+        LLs[g] <- LogLikMstep(pars[[g]], Theta=gTheta[[g]], rs=rlist[[g]],
+                              itemloc=itemloc, NO.CUSTOM=NO.CUSTOM, any.prior=ANY.PRIOR[g])
 #     return(-sum(LLs))
     return(sum(LLs))
 }
@@ -321,16 +341,11 @@ Mstep.grad <- function(p, est, longpars, pars, ngroups, J, gTheta, PrepList, L,
             longpars[constrain[[i]][-1L]] <- longpars[constrain[[i]][1L]]
     pars <- reloadPars(longpars=longpars, pars=pars, ngroups=ngroups, J=J)
     g <- rep(0, ncol(L))
-    ind1 <- 1L
     for(group in 1L:ngroups){
         for (i in 1L:J){
             deriv <- DERIV[[group]][[i]](x=pars[[group]][[i]], Theta=gTheta[[group]], EM=TRUE)
-            ind2 <- ind1 + length(deriv$grad) - 1L
-            g[ind1:ind2] <- deriv$grad
-            ind1 <- ind2 + 1L
+            g[pars[[group]][[i]]@parnum] <- deriv$grad
         }
-        ind2 <- ind1 + length(pars[[group]][[i + 1L]]@par) - 1L
-        ind1 <- ind2 + 1L
     }
     grad <- g %*% L
     return(grad[est])
