@@ -1,11 +1,11 @@
 #' Compute profiled-likelihood confidence intervals
-#' 
-#' Computes profiled-likelihood based confidence intervals. Supports the inclusion of prior parameter 
-#' distributions as well as equality constraints. For multidimensional models, the CI's for the slopes 
-#' are not estimated due to the possibility of signs flipping during estimation. In unidimensional models, the 
+#'
+#' Computes profiled-likelihood based confidence intervals. Supports the inclusion of prior parameter
+#' distributions as well as equality constraints. For multidimensional models, the CI's for the slopes
+#' are not estimated due to the possibility of signs flipping during estimation. In unidimensional models, the
 #' slope parameters are assumed to be greater than zero, and a lower bound is imposed to ensure that sign
 #' flipping does not occur.
-#' 
+#'
 #' @aliases PLCI.mirt
 #' @param mod a converged mirt model
 #' @param alpha two-tailed alpha critical level
@@ -15,37 +15,37 @@
 #' @export PLCI.mirt
 #' @seealso
 #' \code{\link{boot.mirt}}
-#' 
+#'
 #' @examples
 #'
 #' \dontrun{
 #' mirtCluster() #use all available cores to estimate CI's in parallel
 #' dat <- expand.table(LSAT7)
 #' mod <- mirt(dat, 1)
-#' 
+#'
 #' result <- PLCI.mirt(mod)
 #' result
-#' 
+#'
 #' mod2 <- mirt(Science, 1)
 #' result2 <- PLCI.mirt(mod2)
 #' result2
-#' 
+#'
 #' #only estimate CI's slopes
 #' sv <- mod2values(mod2)
 #' parnum <- sv$parnum[sv$name == 'a1']
 #' result3 <- PLCI.mirt(mod2, parnum=parnum)
 #' result3
-#' 
+#'
 #' }
 PLCI.mirt <- function(mod, alpha = .05, parnum = NULL){
-    
-    compute.LL <- function(dat, model, sv, large, parprior){        
+
+    compute.LL <- function(dat, model, sv, large, parprior){
         tmpmod <- suppressMessages(mirt::mirt(dat, model, pars = sv, verbose = FALSE, parprior=parprior,
                                         large=large, calcNull=FALSE))
         tmpmod@logLik
     }
-    
-    f.min <- function(value, dat, model, which, sv, get.LL, large, parprior, parnames){        
+
+    f.min <- function(value, dat, model, which, sv, get.LL, large, parprior, parnames){
         sv$est[which] <- FALSE
         sv$value[which] <- value
         if(sv$class[which] == 'graded'){
@@ -59,7 +59,7 @@ PLCI.mirt <- function(mod, alpha = .05, parnum = NULL){
                     ds <- c(seq(from = value+1, to=value+.1, length.out=nrow(itemsv)-1L), value)
                 } else {
                     ds <- c(seq(from = value+1, to=value+.1, length.out=itemnum-itemsv$parnum[1L]),
-                            value, seq(from = value-.1, to=value-1, 
+                            value, seq(from = value-.1, to=value-1,
                                        length.out=itemsv$parnum[nrow(itemsv)]-itemnum))
                 }
                 sv$value[itemsv$parnum] <- ds
@@ -71,27 +71,27 @@ PLCI.mirt <- function(mod, alpha = .05, parnum = NULL){
         attr(ret, 'value') <- value
         ret
     }
-    
+
     LLpar <- function(parnum, parnums, parnames, lbound, ubound, dat, model, large, sv, get.LL, parprior){
         lower <- ifelse(lbound[parnum] == -Inf, -10, lbound[parnum])
         upper <- ifelse(ubound[parnum] == Inf, 10, ubound[parnum])
         mid <- pars[parnum]
         if(mid > lower){
             opt.lower <- optimize(f.min, lower = lower, upper = mid, dat=dat, model=model, large=large,
-                                  which=parnums[parnum], sv=sv, get.LL=get.LL, parprior=parprior, 
+                                  which=parnums[parnum], sv=sv, get.LL=get.LL, parprior=parprior,
                                   parnames=parnames, tol = .01)
         } else opt.lower <- list(minimum = lower)
         if(mid < upper){
             opt.upper <- optimize(f.min, lower = mid, upper = upper, dat=dat, model=model, large=large,
-                                  which=parnums[parnum], sv=sv, get.LL=get.LL, parprior=parprior, 
+                                  which=parnums[parnum], sv=sv, get.LL=get.LL, parprior=parprior,
                                   parnames=parnames, tol = .01)
         } else opt.upper <- list(minimum = upper)
         c(lower=opt.lower$minimum, upper=opt.upper$minimum)
     }
-    
+
     dat <- mod@data
-    model <- mod@model[[1L]]    
-    parprior <- mod@parprior 
+    model <- mod@model[[1L]]
+    parprior <- mod@parprior
     if(length(parprior) == 0L) parprior <- NULL
     sv <- mod2values(mod)
     large <- mirt(mod@data, mod@model[[1L]], large = TRUE)
@@ -99,7 +99,7 @@ PLCI.mirt <- function(mod, alpha = .05, parnum = NULL){
     sv$lbound[sv$name == 'a1'] <- 0
     if(!is.null(parnum)){
         tmp <- sv$parnum %in% parnum
-        pars <- sv$value[tmp]    
+        pars <- sv$value[tmp]
         LL.upper.crit <- LL.lower.crit <- pars
         parnums <- sv$parnum[tmp]
         itemtypes <- sv$class[tmp]
@@ -107,14 +107,14 @@ PLCI.mirt <- function(mod, alpha = .05, parnum = NULL){
         lbound <- sv$lbound[tmp]
         ubound <- sv$ubound[tmp]
     } else {
-        pars <- sv$value    
-        pars <- LL.upper.crit <- LL.lower.crit <- pars[sv$est]    
+        pars <- sv$value
+        pars <- LL.upper.crit <- LL.lower.crit <- pars[sv$est]
         parnums <- sv$parnum[sv$est]
         itemtypes <- sv$class[sv$est]
         parnames <- sv$name[sv$est]
         lbound <- sv$lbound[sv$est]
-        ubound <- sv$ubound[sv$est]        
-    }    
+        ubound <- sv$ubound[sv$est]
+    }
     if(mod@nfact > 1L && is.null(parnum)){
         #don't estimate slopes of multidimensional
         tmp <- !(sv$name %in% paste0('a', 1:20)) & sv$est
@@ -127,17 +127,17 @@ PLCI.mirt <- function(mod, alpha = .05, parnum = NULL){
     }
     LL <- mod@logLik
     get.LL <- LL - qchisq(1-alpha, 1)/2
-    if(!is.null(globalenv()$MIRTCLUSTER)){
-        result <- t(parallel::parSapply(cl=globalenv()$MIRTCLUSTER, 1L:length(parnums), LLpar, 
-                                        parnums=parnums, parnames=parnames, lbound=lbound, ubound=ubound, 
+    if(!is.null(mirtClusterEnv$MIRTCLUSTER)){
+        result <- t(parallel::parSapply(cl=mirtClusterEnv$MIRTCLUSTER, 1L:length(parnums), LLpar,
+                                        parnums=parnums, parnames=parnames, lbound=lbound, ubound=ubound,
                                         dat=dat, model=model,
                                         large=large, sv=sv, get.LL=get.LL, parprior=parprior))
     } else {
-        result <- t(sapply(1L:length(parnums), LLpar, parnums=parnums, parnames=parnames, lbound=lbound, 
-                           ubound=ubound, dat=dat, model=model, large=large, sv=sv, get.LL=get.LL, parprior=parprior))           
+        result <- t(sapply(1L:length(parnums), LLpar, parnums=parnums, parnames=parnames, lbound=lbound,
+                           ubound=ubound, dat=dat, model=model, large=large, sv=sv, get.LL=get.LL, parprior=parprior))
     }
-    colnames(result) <- c(paste0('lower_', alpha/2*100), paste0('upper_', (1-alpha/2)*100)) 
-    ret <- data.frame(Item=sv$item[parnums], class=itemtypes, parnam=sv$name[parnums], value=pars, 
+    colnames(result) <- c(paste0('lower_', alpha/2*100), paste0('upper_', (1-alpha/2)*100))
+    ret <- data.frame(Item=sv$item[parnums], class=itemtypes, parnam=sv$name[parnums], value=pars,
                       result, row.names=NULL)
     ret
 }
